@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
-* Copyright (C) 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 - 2017 Jan Bajer aka bajasoft <jbajer@gmail.com>
 * Copyright (C) 2016 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -11,11 +11,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -30,7 +30,7 @@
 namespace Otter
 {
 
-class AddressCompletionModel : public QAbstractListModel
+class AddressCompletionModel final : public QAbstractListModel
 {
 	Q_OBJECT
 
@@ -40,67 +40,75 @@ public:
 		UnknownCompletionType = 0,
 		BookmarksCompletionType = 1,
 		HistoryCompletionType = 2,
-		SearchSuggestionsCompletionType = 4,
-		SpecialPagesCompletionType = 8,
-		LocalPathSuggestionsCompletionType = 16
+		TypedHistoryCompletionType = 4,
+		SearchSuggestionsCompletionType = 8,
+		SpecialPagesCompletionType = 16,
+		LocalPathSuggestionsCompletionType = 32
 	};
 
 	Q_DECLARE_FLAGS(CompletionTypes, CompletionType)
 
-	enum EntryType
-	{
-		UnknownType = 0,
-		HeaderType,
-		BookmarkType,
-		HistoryType,
-		TypedInHistoryType,
-		SearchSuggestionType,
-		SpecialPageType,
-		LocalPathType
-	};
-
 	enum EntryRole
 	{
 		TextRole = Qt::DisplayRole,
 		UrlRole = Qt::StatusTipRole,
 		TitleRole = Qt::UserRole,
-		MatchRole = (Qt::UserRole + 1),
-		KeywordRole = (Qt::UserRole + 2),
-		TypeRole = (Qt::UserRole + 3)
+		MatchRole,
+		KeywordRole,
+		TypeRole,
+		TimeVisitedRole
 	};
 
-	struct CompletionEntry
+	struct CompletionEntry final
 	{
+		enum EntryType
+		{
+			UnknownType = 0,
+			HeaderType,
+			BookmarkType,
+			HistoryType,
+			TypedInHistoryType,
+			SearchSuggestionType,
+			SpecialPageType,
+			LocalPathType
+		};
+
 		QString text;
 		QString title;
 		QString match;
 		QString keyword;
 		QUrl url;
 		QIcon icon;
-		EntryType type;
+		QDateTime timeVisited;
+		EntryType type = UnknownType;
 
-		explicit CompletionEntry(const QUrl &urlValue, const QString &titleValue, const QString &matchValue, const QIcon &iconValue, EntryType typeValue) : title(titleValue), match(matchValue), url(urlValue), icon(iconValue), type(typeValue)
+		explicit CompletionEntry(const QUrl &urlValue, const QString &titleValue, const QString &matchValue, const QIcon &iconValue, const QDateTime timeVisitedValue, EntryType typeValue) : title(titleValue), match(matchValue), url(urlValue), icon(iconValue), timeVisited(timeVisitedValue), type(typeValue)
+		{
+		}
+
+		CompletionEntry()
 		{
 		}
 	};
 
 	explicit AddressCompletionModel(QObject *parent = nullptr);
 
-	QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
-	QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
-	Qt::ItemFlags flags(const QModelIndex &index) const;
-	int rowCount(const QModelIndex &index = QModelIndex()) const;
-	bool event(QEvent *event);
+	void setTypes(CompletionTypes types);
+	QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
+	QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const override;
+	Qt::ItemFlags flags(const QModelIndex &index) const override;
+	int rowCount(const QModelIndex &index = {}) const override;
+	bool event(QEvent *event) override;
 
 public slots:
-	void setFilter(const QString &filter = QString());
+	void setFilter(const QString &filter = {});
 
 protected:
-	void timerEvent(QTimerEvent *event);
+	void timerEvent(QTimerEvent *event) override;
 	void updateModel();
 
 private:
-	QList<CompletionEntry> m_completions;
+	QVector<CompletionEntry> m_completions;
 	QString m_filter;
 	SearchEnginesManager::SearchEngineDefinition m_defaultSearchEngine;
 	AddressCompletionModel::CompletionTypes m_types;
