@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -27,19 +27,9 @@
 namespace Otter
 {
 
-struct EventDefinition
-{
-	QString title;
-	QString description;
-	QString playSound;
-	int identifier = -1;
-	bool showAlert = false;
-	bool showNotification = false;
-};
-
 class NotificationsManager;
 
-class Notification : public QObject
+class Notification final : public QObject
 {
 	Q_OBJECT
 
@@ -47,12 +37,12 @@ public:
 	enum NotificationLevel
 	{
 		InformationLevel = 0,
-		WarningLevel = 1,
-		ErrorLevel = 2
+		WarningLevel,
+		ErrorLevel
 	};
 
-	void markClicked();
-	void markIgnored();
+	void markAsClicked();
+	void markAsIgnored();
 	void setData(const QVariant &data);
 	QString getMessage() const;
 	QDateTime getCreationTime() const;
@@ -76,7 +66,7 @@ signals:
 friend class NotificationsManager;
 };
 
-class NotificationsManager : public QObject
+class NotificationsManager final : public QObject
 {
 	Q_OBJECT
 	Q_ENUMS(EventIdentifier)
@@ -84,20 +74,31 @@ class NotificationsManager : public QObject
 public:
 	enum EventIdentifier
 	{
-		TransferCompletedEvent = 0,
-		UpdateAvailableEvent = 1
+		FeedUpdatedEvent = 0,
+		TransferCompletedEvent,
+		UpdateAvailableEvent
 	};
 
-	static void createInstance(QObject *parent = nullptr);
+	struct EventDefinition final
+	{
+		QString title;
+		QString description;
+		QString playSound;
+		int identifier = -1;
+		bool showAlert = false;
+		bool showNotification = false;
+	};
+
+	static void createInstance();
 	static NotificationsManager* getInstance();
 	static Notification* createNotification(int event, const QString &message, Notification::NotificationLevel level = Notification::InformationLevel, QObject *parent = nullptr);
 	static QString getEventName(int identifier);
 	static EventDefinition getEventDefinition(int identifier);
 	static QVector<EventDefinition> getEventDefinitions();
-	static int registerEvent(const QString &title, const QString &description = QString());
+	static int registerEvent(const QString &title, const QString &description = {});
 
 protected:
-	explicit NotificationsManager(QObject *parent = nullptr);
+	explicit NotificationsManager(QObject *parent);
 
 private:
 	static NotificationsManager *m_instance;
