@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,25 +9,22 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "SearchEnginesManager.h"
-#include "NetworkManager.h"
-#include "NetworkManagerFactory.h"
+#include "ItemModel.h"
 #include "SessionsManager.h"
 #include "SettingsManager.h"
 #include "ThemesManager.h"
-#include "Utils.h"
 
-#include <QtCore/QBuffer>
 #include <QtCore/QDir>
-#include <QtCore/QRegularExpression>
+#include <QtCore/QFile>
 #include <QtCore/QXmlStreamReader>
 #include <QtCore/QXmlStreamWriter>
 #include <QtNetwork/QNetworkRequest>
@@ -46,15 +43,15 @@ SearchEnginesManager::SearchEnginesManager(QObject *parent) : QObject(parent)
 {
 }
 
-void SearchEnginesManager::createInstance(QObject *parent)
+void SearchEnginesManager::createInstance()
 {
 	if (!m_instance)
 	{
-		m_instance = new SearchEnginesManager(parent);
+		m_instance = new SearchEnginesManager(QCoreApplication::instance());
 	}
 }
 
-void SearchEnginesManager::initialize()
+void SearchEnginesManager::ensureInitialized()
 {
 	if (!m_isInitialized)
 	{
@@ -62,30 +59,25 @@ void SearchEnginesManager::initialize()
 
 		loadSearchEngines();
 
-		connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), m_instance, SLOT(optionChanged(int)));
-	}
-}
-
-void SearchEnginesManager::optionChanged(int identifier)
-{
-	if (identifier == SettingsManager::Search_SearchEnginesOrderOption)
-	{
-		loadSearchEngines();
+		connect(SettingsManager::getInstance(), &SettingsManager::optionChanged, m_instance, &SearchEnginesManager::handleOptionChanged);
 	}
 }
 
 void SearchEnginesManager::loadSearchEngines()
 {
-	m_searchEngines.clear();
-	m_searchKeywords.clear();
+	m_searchEnginesOrder = SettingsManager::getOption(SettingsManager::Search_SearchEnginesOrderOption).toStringList();
 
-	m_searchEnginesOrder = SettingsManager::getValue(SettingsManager::Search_SearchEnginesOrderOption).toStringList();
+	m_searchEngines.clear();
+	m_searchEngines.reserve(m_searchEnginesOrder.count());
+
+	m_searchKeywords.clear();
+	m_searchKeywords.reserve(m_searchEnginesOrder.count());
 
 	const QStringList searchEnginesOrder(m_searchEnginesOrder);
 
 	for (int i = 0; i < searchEnginesOrder.count(); ++i)
 	{
-		QFile file(SessionsManager::getReadableDataPath(QLatin1String("searches/") + searchEnginesOrder.at(i) + QLatin1String(".xml")));
+		QFile file(SessionsManager::getReadableDataPath(QLatin1String("searchEngines/") + searchEnginesOrder.at(i) + QLatin1String(".xml")));
 
 		if (!file.open(QIODevice::ReadOnly))
 		{
@@ -94,50 +86,33 @@ void SearchEnginesManager::loadSearchEngines()
 			continue;
 		}
 
-		SearchEngineDefinition searchEngine(loadSearchEngine(&file, searchEnginesOrder.at(i), true));
+		const SearchEngineDefinition searchEngine(loadSearchEngine(&file, searchEnginesOrder.at(i), true));
 
 		file.close();
 
-		if (searchEngine.identifier.isEmpty())
-		{
-			m_searchEnginesOrder.removeAll(searchEnginesOrder.at(i));
-		}
-		else
+		if (searchEngine.isValid())
 		{
 			m_searchEngines[searchEnginesOrder.at(i)] = searchEngine;
 		}
+		else
+		{
+			m_searchEnginesOrder.removeAll(searchEnginesOrder.at(i));
+		}
 	}
 
+	m_searchEngines.squeeze();
+
 	emit m_instance->searchEnginesModified();
 
 	updateSearchEnginesModel();
 	updateSearchEnginesOptions();
 }
 
-void SearchEnginesManager::addSearchEngine(const SearchEngineDefinition &searchEngine, bool isDefault)
+void SearchEnginesManager::handleOptionChanged(int identifier)
 {
-	if (!saveSearchEngine(searchEngine))
+	if (identifier == SettingsManager::Search_SearchEnginesOrderOption)
 	{
-		return;
-	}
-
-	if (isDefault)
-	{
-		SettingsManager::setValue(SettingsManager::Search_DefaultSearchEngineOption, searchEngine.identifier);
-	}
-
-	if (m_searchEnginesOrder.contains(searchEngine.identifier))
-	{
-		emit m_instance->searchEnginesModified();
-
-		updateSearchEnginesModel();
-		updateSearchEnginesOptions();
-	}
-	else
-	{
-		m_searchEnginesOrder.append(searchEngine.identifier);
-
-		SettingsManager::setValue(SettingsManager::Search_SearchEnginesOrderOption, m_searchEnginesOrder);
+		loadSearchEngines();
 	}
 }
 
@@ -156,9 +131,9 @@ void SearchEnginesManager::updateSearchEnginesModel()
 	{
 		const SearchEngineDefinition search(getSearchEngine(searchEngines.at(i)));
 
-		if (!search.identifier.isEmpty())
+		if (search.isValid())
 		{
-			QStandardItem *item(new QStandardItem((search.icon.isNull() ? ThemesManager::getIcon(QLatin1String("edit-find")) : search.icon), QString()));
+			QStandardItem *item(new QStandardItem((search.icon.isNull() ? ThemesManager::createIcon(QLatin1String("edit-find")) : search.icon), {}));
 			item->setData(search.title, TitleRole);
 			item->setData(search.identifier, IdentifierRole);
 			item->setData(search.keyword, KeywordRole);
@@ -170,15 +145,11 @@ void SearchEnginesManager::updateSearchEnginesModel()
 
 	if (searchEngines.count() > 0)
 	{
-		QStandardItem *separatorItem(new QStandardItem());
-		separatorItem->setData(QLatin1String("separator"), Qt::AccessibleDescriptionRole);
-		separatorItem->setFlags(Qt::ItemIsEnabled | Qt::ItemNeverHasChildren);
-
-		QStandardItem *manageItem(new QStandardItem(ThemesManager::getIcon(QLatin1String("configure")), tr("Manage Search Engines…")));
+		QStandardItem *manageItem(new QStandardItem(ThemesManager::createIcon(QLatin1String("configure")), tr("Manage Search Engines…")));
 		manageItem->setData(QLatin1String("configure"), Qt::AccessibleDescriptionRole);
 		manageItem->setFlags(manageItem->flags() | Qt::ItemNeverHasChildren);
 
-		m_searchEnginesModel->appendRow(separatorItem);
+		m_searchEnginesModel->appendRow(new ItemModel::Item(ItemModel::SeparatorType));
 		m_searchEnginesModel->appendRow(manageItem);
 	}
 
@@ -188,11 +159,21 @@ void SearchEnginesManager::updateSearchEnginesModel()
 void SearchEnginesManager::updateSearchEnginesOptions()
 {
 	const QStringList searchEngines(m_searchEngines.keys());
+	QVector<SettingsManager::OptionDefinition::Choice> searchEngineChoices;
+	searchEngineChoices.reserve(searchEngines.count());
+
+	for (int i = 0; i < searchEngines.count(); ++i)
+	{
+		const SearchEngineDefinition searchEngine(getSearchEngine(searchEngines.at(i)));
+
+		searchEngineChoices.append({(searchEngine.title.isEmpty() ? tr("Unknown") : searchEngine.title), searchEngines.at(i), searchEngine.icon});
+	}
+
 	SettingsManager::OptionDefinition defaultQuickSearchEngineOption(SettingsManager::getOptionDefinition(SettingsManager::Search_DefaultQuickSearchEngineOption));
-	defaultQuickSearchEngineOption.choices = searchEngines;
+	defaultQuickSearchEngineOption.choices = searchEngineChoices;
 
 	SettingsManager::OptionDefinition defaultSearchEngineOption(SettingsManager::getOptionDefinition(SettingsManager::Search_DefaultSearchEngineOption));
-	defaultSearchEngineOption.choices = searchEngines;
+	defaultSearchEngineOption.choices = searchEngineChoices;
 
 	SettingsManager::updateOptionDefinition(SettingsManager::Search_DefaultQuickSearchEngineOption, defaultQuickSearchEngineOption);
 	SettingsManager::updateOptionDefinition(SettingsManager::Search_DefaultSearchEngineOption, defaultSearchEngineOption);
@@ -206,20 +187,12 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 	}
 
 	QString urlString(searchUrl.url);
-	QHash<QString, QString> values;
-	values[QLatin1String("searchTerms")] = query;
-	values[QLatin1String("count")] = QString();
-	values[QLatin1String("startIndex")] = QString();
-	values[QLatin1String("startPage")] = QString();
-	values[QLatin1String("language")] = QLocale::system().name();
-	values[QLatin1String("inputEncoding")] = QLatin1String("UTF-8");
-	values[QLatin1String("outputEncoding")] = QLatin1String("UTF-8");
+	const QHash<QString, QString> values({{QLatin1String("searchTerms"), query}, {QLatin1String("count"), QString()}, {QLatin1String("startIndex"), QString()}, {QLatin1String("startPage"), QString()}, {QLatin1String("language"), QLocale::system().name().replace(QLatin1Char('_'), QLatin1Char('-'))}, {QLatin1String("inputEncoding"), QLatin1String("UTF-8")}, {QLatin1String("outputEncoding"), QLatin1String("UTF-8")}});
+	QHash<QString, QString>::const_iterator iterator;
 
-	QHash<QString, QString>::iterator valuesIterator;
-
-	for (valuesIterator = values.begin(); valuesIterator != values.end(); ++valuesIterator)
+	for (iterator = values.constBegin(); iterator != values.constEnd(); ++iterator)
 	{
-		urlString = urlString.replace(QStringLiteral("{%1}").arg(valuesIterator.key()), QUrl::toPercentEncoding(valuesIterator.value()));
+		urlString = urlString.replace(QLatin1Char('{') + iterator.key() + QLatin1Char('}'), QUrl::toPercentEncoding(iterator.value()));
 	}
 
 	*method = ((searchUrl.method == QLatin1String("post")) ? QNetworkAccessManager::PostOperation : QNetworkAccessManager::GetOperation);
@@ -231,12 +204,7 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 
 	for (int i = 0; i < parameters.count(); ++i)
 	{
-		QString value(parameters.at(i).second);
-
-		for (valuesIterator = values.begin(); valuesIterator != values.end(); ++valuesIterator)
-		{
-			value = value.replace(QStringLiteral("{%1}").arg(valuesIterator.key()), valuesIterator.value());
-		}
+		const QString value(Utils::substitutePlaceholders(parameters.at(i).second, values));
 
 		if (*method == QNetworkAccessManager::GetOperation)
 		{
@@ -256,7 +224,7 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 
 				for (int j = 0; j < plainValue.length(); ++j)
 				{
-					const char character(plainValue[j]);
+					const char character(plainValue.at(j));
 
 					if (character == 32 || (character >= 33 && character <= 126 && character != 61))
 					{
@@ -265,8 +233,8 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 					else
 					{
 						encodedValue.append(QLatin1Char('='));
-						encodedValue.append(hex[((character >> 4) & 0x0F)]);
-						encodedValue.append(hex[(character & 0x0F)]);
+						encodedValue.append(hex[(character >> 4) & 0x0F]);
+						encodedValue.append(hex[character & 0x0F]);
 					}
 				}
 
@@ -302,7 +270,7 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 	SearchEngineDefinition searchEngine;
 	searchEngine.identifier = identifier;
 
-	QXmlStreamReader reader(device->readAll());
+	QXmlStreamReader reader(device);
 
 	if (reader.readNextStartElement() && reader.name() == QLatin1String("OpenSearchDescription"))
 	{
@@ -314,19 +282,24 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 
 			if (reader.isStartElement())
 			{
+				const QXmlStreamAttributes attributes(reader.attributes());
+
 				if (reader.name() == QLatin1String("Url"))
 				{
-					if (reader.attributes().value(QLatin1String("rel")) == QLatin1String("self") || reader.attributes().value(QLatin1String("type")) == QLatin1String("application/opensearchdescription+xml"))
+					const QString rel(attributes.value(QLatin1String("rel")).toString());
+					const QString type(attributes.value(QLatin1String("type")).toString());
+
+					if (rel == QLatin1String("self") || type == QLatin1String("application/opensearchdescription+xml"))
 					{
-						searchEngine.selfUrl = QUrl(reader.attributes().value(QLatin1String("template")).toString());
+						searchEngine.selfUrl = QUrl(attributes.value(QLatin1String("template")).toString());
 
 						currentUrl = nullptr;
 					}
-					else if (reader.attributes().value(QLatin1String("rel")) == QLatin1String("suggestions") || reader.attributes().value(QLatin1String("type")) == QLatin1String("application/x-suggestions+json"))
+					else if ((rel == QLatin1String("suggestions") && type != QLatin1String("x-suggestions+xm")) || type == QLatin1String("application/x-suggestions+json"))
 					{
 						currentUrl = &searchEngine.suggestionsUrl;
 					}
-					else if (!reader.attributes().hasAttribute(QLatin1String("rel")) || reader.attributes().value(QLatin1String("rel")) == QLatin1String("results"))
+					else if ((rel.isEmpty() || rel == QLatin1String("results")) && !type.contains(QLatin1String("suggestions")))
 					{
 						currentUrl = &searchEngine.resultsUrl;
 					}
@@ -337,14 +310,14 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 
 					if (currentUrl)
 					{
-						currentUrl->url = reader.attributes().value(QLatin1String("template")).toString();
-						currentUrl->enctype = reader.attributes().value(QLatin1String("enctype")).toString().toLower();
-						currentUrl->method = reader.attributes().value(QLatin1String("method")).toString().toLower();
+						currentUrl->url = attributes.value(QLatin1String("template")).toString();
+						currentUrl->enctype = attributes.value(QLatin1String("enctype")).toString().toLower();
+						currentUrl->method = attributes.value(QLatin1String("method")).toString().toLower();
 					}
 				}
-				else if ((reader.name() == QLatin1String("Param") || reader.name() == QLatin1String("Parameter")) && currentUrl)
+				else if (currentUrl && (reader.name() == QLatin1String("Param") || reader.name() == QLatin1String("Parameter")))
 				{
-					currentUrl->parameters.addQueryItem(reader.attributes().value(QLatin1String("name")).toString(), reader.attributes().value(QLatin1String("value")).toString());
+					currentUrl->parameters.addQueryItem(attributes.value(QLatin1String("name")).toString(), attributes.value(QLatin1String("value")).toString());
 				}
 				else if (reader.name() == QLatin1String("Shortcut"))
 				{
@@ -380,9 +353,9 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 				{
 					const QString data(reader.readElementText());
 
-					if (data.startsWith(QLatin1String("data:image")))
+					if (data.startsWith(QLatin1String("data:image/")))
 					{
-						searchEngine.icon = QIcon(QPixmap::fromImage(QImage::fromData(QByteArray::fromBase64(data.mid(data.indexOf(QLatin1String("base64,")) + 7).toUtf8()), "png")));
+						searchEngine.icon = QIcon(Utils::loadPixmapFromDataUri(data));
 					}
 					else
 					{
@@ -398,7 +371,7 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 	}
 	else
 	{
-		searchEngine.identifier = QString();
+		searchEngine.identifier.clear();
 	}
 
 	return searchEngine;
@@ -411,16 +384,13 @@ SearchEnginesManager* SearchEnginesManager::getInstance()
 
 QStandardItemModel* SearchEnginesManager::getSearchEnginesModel()
 {
-	if (!m_isInitialized)
-	{
-		initialize();
-	}
+	ensureInitialized();
 
 	if (!m_searchEnginesModel)
 	{
 		m_searchEnginesModel = new QStandardItemModel(m_instance);
 
-		m_instance->updateSearchEnginesModel();
+		updateSearchEnginesModel();
 	}
 
 	return m_searchEnginesModel;
@@ -428,10 +398,7 @@ QStandardItemModel* SearchEnginesManager::getSearchEnginesModel()
 
 SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::getSearchEngine(const QString &identifier, bool byKeyword)
 {
-	if (!m_isInitialized)
-	{
-		initialize();
-	}
+	ensureInitialized();
 
 	if (byKeyword)
 	{
@@ -453,7 +420,24 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::getSearchEngi
 
 	if (identifier.isEmpty())
 	{
-		return m_searchEngines.value(SettingsManager::getValue(SettingsManager::Search_DefaultSearchEngineOption).toString(), SearchEngineDefinition());
+		return m_searchEngines.value(SettingsManager::getOption(SettingsManager::Search_DefaultSearchEngineOption).toString(), SearchEngineDefinition());
+	}
+
+	if (!m_searchEngines.contains(identifier))
+	{
+		QFile file(SessionsManager::getReadableDataPath(QLatin1String("searchEngines/") + identifier + QLatin1String(".xml")));
+
+		if (file.open(QIODevice::ReadOnly))
+		{
+			const SearchEnginesManager::SearchEngineDefinition searchEngine(SearchEnginesManager::loadSearchEngine(&file, identifier));
+
+			if (searchEngine.isValid())
+			{
+				m_searchEngines[identifier] = searchEngine;
+			}
+
+			file.close();
+		}
 	}
 
 	return m_searchEngines.value(identifier, SearchEngineDefinition());
@@ -461,20 +445,14 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::getSearchEngi
 
 QStringList SearchEnginesManager::getSearchEngines()
 {
-	if (!m_isInitialized)
-	{
-		initialize();
-	}
+	ensureInitialized();
 
 	return m_searchEnginesOrder;
 }
 
 QStringList SearchEnginesManager::getSearchKeywords()
 {
-	if (!m_isInitialized)
-	{
-		initialize();
-	}
+	ensureInitialized();
 
 	return m_searchKeywords;
 }
@@ -486,10 +464,7 @@ bool SearchEnginesManager::hasSearchEngine(const QUrl &url)
 		return false;
 	}
 
-	if (!m_isInitialized)
-	{
-		initialize();
-	}
+	ensureInitialized();
 
 	QHash<QString, SearchEngineDefinition>::iterator iterator;
 
@@ -504,18 +479,49 @@ bool SearchEnginesManager::hasSearchEngine(const QUrl &url)
 	return false;
 }
 
-bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &searchEngine)
+bool SearchEnginesManager::addSearchEngine(const SearchEngineDefinition &searchEngine)
 {
-	if (SessionsManager::isReadOnly() || searchEngine.identifier.isEmpty())
+	if (!saveSearchEngine(searchEngine))
 	{
 		return false;
 	}
 
-	QDir().mkpath(SessionsManager::getWritableDataPath(QLatin1String("searches")));
+	if (m_searchEnginesOrder.contains(searchEngine.identifier))
+	{
+		emit m_instance->searchEnginesModified();
 
-	QFile file(SessionsManager::getWritableDataPath(QLatin1String("searches/") + searchEngine.identifier + QLatin1String(".xml")));
+		updateSearchEnginesModel();
+		updateSearchEnginesOptions();
+	}
+	else
+	{
+		m_searchEnginesOrder.append(searchEngine.identifier);
 
-	if (!file.open(QFile::WriteOnly))
+		SettingsManager::setOption(SettingsManager::Search_SearchEnginesOrderOption, m_searchEnginesOrder);
+	}
+
+	return true;
+}
+
+bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &searchEngine)
+{
+	if (SessionsManager::isReadOnly())
+	{
+		return false;
+	}
+
+	QString identifier(searchEngine.identifier);
+
+	if (identifier.isEmpty())
+	{
+		identifier = searchEngine.createIdentifier();
+	}
+
+	QDir().mkpath(SessionsManager::getWritableDataPath(QLatin1String("searchEngines")));
+
+	QFile file(SessionsManager::getWritableDataPath(QLatin1String("searchEngines/") + identifier + QLatin1String(".xml")));
+
+	if (!file.open(QIODevice::WriteOnly))
 	{
 		return false;
 	}
@@ -540,17 +546,12 @@ bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &search
 	if (!searchEngine.icon.isNull())
 	{
 		const QSize size(searchEngine.icon.availableSizes().value(0, QSize(16, 16)));
-		QByteArray data;
-		QBuffer buffer(&data);
-		buffer.open(QIODevice::WriteOnly);
-
-		searchEngine.icon.pixmap(size).save(&buffer, "PNG");
 
 		writer.writeStartElement(QLatin1String("Image"));
 		writer.writeAttribute(QLatin1String("width"), QString::number(size.width()));
 		writer.writeAttribute(QLatin1String("height"), QString::number(size.height()));
 		writer.writeAttribute(QLatin1String("type"), QLatin1String("image/png"));
-		writer.writeCharacters(QStringLiteral("data:image/png;base64,%1").arg(QString(data.toBase64())));
+		writer.writeCharacters(Utils::savePixmapAsDataUri(searchEngine.icon.pixmap(size)));
 		writer.writeEndElement();
 	}
 
@@ -618,115 +619,72 @@ bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &search
 	return true;
 }
 
-bool SearchEnginesManager::setupSearchQuery(const QString &query, const QString &searchEngine, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body)
+bool SearchEnginesManager::setupSearchQuery(const QString &query, const QString &identifier, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body)
 {
-	const SearchEngineDefinition search(getSearchEngine(searchEngine));
+	const SearchEngineDefinition searchEngine(getSearchEngine(identifier));
 
-	if (search.identifier.isEmpty())
+	if (searchEngine.isValid())
 	{
-		return false;
+		setupQuery(query, searchEngine.resultsUrl, request, method, body);
+
+		return true;
 	}
 
-	setupQuery(query, search.resultsUrl, request, method, body);
-
-	return true;
+	return false;
 }
 
-SearchEngineFetchJob::SearchEngineFetchJob(const QUrl &url, const QString &identifier, bool saveSearchEngine, QObject *parent) : QObject(parent),
-	m_reply(nullptr),
-	m_isFetchingIcon(false),
+SearchEngineFetchJob::SearchEngineFetchJob(const QUrl &url, const QString &identifier, bool saveSearchEngine, QObject *parent) : FetchJob(url, parent),
 	m_needsToSaveSearchEngine(saveSearchEngine)
 {
-	m_searchEngine.identifier = (identifier.isEmpty() ? Utils::createIdentifier(QString(), SearchEnginesManager::getSearchEngines()) : identifier);
-
-	QNetworkRequest request(url);
-	request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
-
-	m_reply = NetworkManagerFactory::getNetworkManager()->get(request);
-
-	connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(handleRequestFailed()));
-	connect(m_reply, SIGNAL(finished()), this, SLOT(handleRequestFinished()));
+	m_searchEngine.identifier = identifier;
 }
 
-SearchEngineFetchJob::~SearchEngineFetchJob()
+SearchEnginesManager::SearchEngineDefinition SearchEngineFetchJob::getSearchEngine() const
 {
-	m_reply->deleteLater();
+	return m_searchEngine;
 }
 
-void SearchEngineFetchJob::cancel()
+void SearchEngineFetchJob::handleSuccessfulReply(QNetworkReply *reply)
 {
-	if (m_reply)
-	{
-		m_reply->abort();
-	}
-
-	emit jobFinished(false);
-}
-
-void SearchEngineFetchJob::handleRequestFailed()
-{
-	deleteLater();
-
-	if (m_isFetchingIcon)
-	{
-		SearchEnginesManager::addSearchEngine(m_searchEngine);
-	}
-
-	emit jobFinished(m_isFetchingIcon);
-}
-
-void SearchEngineFetchJob::handleRequestFinished()
-{
-	if (m_isFetchingIcon)
-	{
-		QPixmap pixmap;
-
-		if (pixmap.loadFromData(m_reply->readAll()))
-		{
-			m_searchEngine.icon = QIcon(pixmap);
-		}
-
-		if (m_needsToSaveSearchEngine)
-		{
-			SearchEnginesManager::addSearchEngine(m_searchEngine);
-		}
-
-		deleteLater();
-
-		emit jobFinished(true);
-
-		return;
-	}
-
-	m_searchEngine = SearchEnginesManager::loadSearchEngine(m_reply, m_searchEngine.identifier);
-
 	if (m_searchEngine.identifier.isEmpty())
 	{
-		deleteLater();
+		m_searchEngine.identifier = m_searchEngine.createIdentifier();
+	}
 
-		emit jobFinished(false);
+	m_searchEngine = SearchEnginesManager::loadSearchEngine(reply, m_searchEngine.identifier);
+
+	if (!m_searchEngine.isValid())
+	{
+		markAsFailure();
 
 		return;
 	}
 
 	if (m_searchEngine.selfUrl.isEmpty())
 	{
-		m_searchEngine.selfUrl = m_reply->request().url();
+		m_searchEngine.selfUrl = reply->request().url();
 	}
 
 	if (m_searchEngine.iconUrl.isValid())
 	{
-		m_isFetchingIcon = true;
+		IconFetchJob *job(new IconFetchJob(m_searchEngine.iconUrl, this));
 
-		m_reply->deleteLater();
+		connect(job, &IconFetchJob::jobFinished, this, [=]()
+		{
+			m_searchEngine.icon = job->getIcon();
 
-		QNetworkRequest request(m_searchEngine.iconUrl);
-		request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
+			if (m_needsToSaveSearchEngine)
+			{
+				SearchEnginesManager::addSearchEngine(m_searchEngine);
+			}
 
-		m_reply = NetworkManagerFactory::getNetworkManager()->get(request);
+			markAsFinished();
+			deleteLater();
 
-		connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(handleRequestFailed()));
-		connect(m_reply, SIGNAL(finished()), this, SLOT(handleRequestFinished()));
+			emit jobFinished(true);
+		});
+
+		job->start();
 	}
 	else
 	{
@@ -735,15 +693,11 @@ void SearchEngineFetchJob::handleRequestFinished()
 			SearchEnginesManager::addSearchEngine(m_searchEngine);
 		}
 
+		markAsFinished();
 		deleteLater();
 
 		emit jobFinished(true);
 	}
 }
 
-SearchEnginesManager::SearchEngineDefinition SearchEngineFetchJob::getSearchEngine() const
-{
-	return m_searchEngine;
-}
-
 }
