@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,28 +9,29 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #ifndef OTTER_SEARCHENGINESMANAGER_H
 #define OTTER_SEARCHENGINESMANAGER_H
 
-#include <QtCore/QFile>
+#include "Job.h"
+#include "Utils.h"
+
 #include <QtCore/QUrlQuery>
 #include <QtGui/QIcon>
 #include <QtGui/QStandardItemModel>
 #include <QtNetwork/QNetworkAccessManager>
-#include <QtNetwork/QNetworkReply>
 
 namespace Otter
 {
 
-class SearchEnginesManager : public QObject
+class SearchEnginesManager final : public QObject
 {
 	Q_OBJECT
 
@@ -42,7 +43,7 @@ public:
 		KeywordRole
 	};
 
-	struct SearchUrl
+	struct SearchUrl final
 	{
 		QString url;
 		QString enctype;
@@ -50,7 +51,7 @@ public:
 		QUrlQuery parameters;
 	};
 
-	struct SearchEngineDefinition
+	struct SearchEngineDefinition final
 	{
 		QString identifier;
 		QString title;
@@ -63,31 +64,41 @@ public:
 		SearchUrl resultsUrl;
 		SearchUrl suggestionsUrl;
 		QIcon icon;
+
+		QString createIdentifier(const QStringList &exclude = {}) const
+		{
+			return Utils::createIdentifier(QUrl(resultsUrl.url).host(), (exclude.isEmpty() ? SearchEnginesManager::getSearchEngines() : exclude));
+		}
+
+		bool isValid() const
+		{
+			return !identifier.isEmpty();
+		}
 	};
 
-	static void createInstance(QObject *parent = nullptr);
+	static void createInstance();
 	static void loadSearchEngines();
-	static void addSearchEngine(const SearchEngineDefinition &searchEngine, bool isDefault = false);
 	static void setupQuery(const QString &query, const SearchUrl &searchUrl, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body);
 	static SearchEngineDefinition loadSearchEngine(QIODevice *device, const QString &identifier, bool checkKeyword = true);
 	static SearchEnginesManager* getInstance();
 	static QStandardItemModel* getSearchEnginesModel();
-	static SearchEngineDefinition getSearchEngine(const QString &identifier = QString(), bool byKeyword = false);
+	static SearchEngineDefinition getSearchEngine(const QString &identifier = {}, bool byKeyword = false);
 	static QStringList getSearchEngines();
 	static QStringList getSearchKeywords();
 	static bool hasSearchEngine(const QUrl &url);
+	static bool addSearchEngine(const SearchEngineDefinition &searchEngine);
 	static bool saveSearchEngine(const SearchEngineDefinition &searchEngine);
-	static bool setupSearchQuery(const QString &query, const QString &searchEngine, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body);
+	static bool setupSearchQuery(const QString &query, const QString &identifier, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body);
 
 protected:
-	explicit SearchEnginesManager(QObject *parent = nullptr);
+	explicit SearchEnginesManager(QObject *parent);
 
-	static void initialize();
+	static void ensureInitialized();
 	static void updateSearchEnginesModel();
 	static void updateSearchEnginesOptions();
 
 protected slots:
-	void optionChanged(int identifier);
+	void handleOptionChanged(int identifier);
 
 private:
 	static SearchEnginesManager *m_instance;
@@ -102,31 +113,21 @@ signals:
 	void searchEnginesModelModified();
 };
 
-class SearchEngineFetchJob : public QObject
+class SearchEngineFetchJob final : public FetchJob
 {
 	Q_OBJECT
 
 public:
-	explicit SearchEngineFetchJob(const QUrl &url, const QString &identifier = QString(), bool saveSearchEngine = true, QObject *parent = nullptr);
-	~SearchEngineFetchJob();
+	explicit SearchEngineFetchJob(const QUrl &url, const QString &identifier = {}, bool saveSearchEngine = true, QObject *parent = nullptr);
 
 	SearchEnginesManager::SearchEngineDefinition getSearchEngine() const;
 
-public slots:
-	void cancel();
-
-protected slots:
-	void handleRequestFailed();
-	void handleRequestFinished();
+protected:
+	void handleSuccessfulReply(QNetworkReply *reply) override;
 
 private:
-	QNetworkReply *m_reply;
 	SearchEnginesManager::SearchEngineDefinition m_searchEngine;
-	bool m_isFetchingIcon;
 	bool m_needsToSaveSearchEngine;
-
-signals:
-	void jobFinished(bool isSuccess);
 };
 
 }
