@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2017 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -22,44 +23,44 @@
 
 #include <QtCore/QObject>
 #include <QtGui/QStandardItemModel>
-#include <QtNetwork/QNetworkAccessManager>
+#include <QtNetwork/QNetworkReply>
 
 namespace Otter
 {
 
-class NetworkManager;
-
-class SearchSuggester : public QObject
+class SearchSuggester final : public QObject
 {
 	Q_OBJECT
 
 public:
-	struct SearchSuggestion
+	struct SearchSuggestion final
 	{
 		QString completion;
 		QString description;
 		QString url;
 	};
 
-	explicit SearchSuggester(const QString &searchEngine, QObject *parent = NULL);
+	explicit SearchSuggester(const QString &searchEngine, QObject *parent = nullptr);
 
 	QStandardItemModel* getModel();
+	QVector<SearchSuggestion> getSuggestions() const;
 
 public slots:
 	void setSearchEngine(const QString &searchEngine);
 	void setQuery(const QString &query);
 
 protected slots:
-	void replyFinished();
+	void handleReplyFinished();
 
 private:
 	QNetworkReply *m_networkReply;
 	QStandardItemModel *m_model;
 	QString m_searchEngine;
 	QString m_query;
+	QVector<SearchSuggestion> m_suggestions;
 
 signals:
-	void suggestionsChanged(const QList<SearchSuggestion> &suggestions);
+	void suggestionsChanged(const QVector<SearchSuggester::SearchSuggestion> &suggestions);
 };
 
 }
