@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2014 - 2015 Piotr Wójcik <chocimier@tlen.pl>
-* Copyright (C) 2014 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2014 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -26,28 +26,25 @@
 namespace Otter
 {
 
-class SessionMainWindow;
-
-class OperaSessionImporter : public Importer
+class OperaSessionImporter final : public Importer
 {
 	Q_OBJECT
 
 public:
-	explicit OperaSessionImporter(QObject *parent = NULL);
+	explicit OperaSessionImporter(QObject *parent = nullptr);
 
-	QWidget* getOptionsWidget();
-	QString getTitle() const;
-	QString getDescription() const;
-	QString getVersion() const;
-	QString getFileFilter() const;
-	QString getSuggestedPath(const QString &path = QString()) const;
-	QString getBrowser() const;
-	QUrl getHomePage() const;
-	QIcon getIcon() const;
-	ImportType getType() const;
+	QWidget* createOptionsWidget(QWidget *parent) override;
+	QString getTitle() const override;
+	QString getDescription() const override;
+	QString getVersion() const override;
+	QString getSuggestedPath(const QString &path = {}) const override;
+	QString getBrowser() const override;
+	QUrl getHomePage() const override;
+	QStringList getFileFilters() const override;
+	ImportType getImportType() const override;
 
 public slots:
-	bool import(const QString &path);
+	bool import(const QString &path) override;
 };
 
 }
