@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2017 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,53 +9,58 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "MenuButtonWidget.h"
 #include "../../../core/SessionsManager.h"
 #include "../../../core/ThemesManager.h"
+#include "../../../ui/MainWindow.h"
 #include "../../../ui/Menu.h"
 #include "../../../ui/ToolBarWidget.h"
 
-#include <QtCore/QFile>
-#include <QtCore/QJsonDocument>
-
 namespace Otter
 {
 
-MenuButtonWidget::MenuButtonWidget(const ActionsManager::ActionEntryDefinition &definition, QWidget *parent) : ToolButtonWidget(definition, parent),
-	m_menu(new Menu(Menu::NoMenuRole, this)),
+MenuButtonWidget::MenuButtonWidget(const ToolBarsManager::ToolBarDefinition::Entry &definition, QWidget *parent) : ToolButtonWidget(definition, parent),
+	m_menu(new Menu(Menu::UnknownMenu, this)),
 	m_isHidden(false)
 {
-	setIcon(ThemesManager::getIcon(QLatin1String("otter-browser"), false));
+	setIcon(ThemesManager::createIcon(QLatin1String("otter-browser"), false));
 	setText(definition.options.value(QLatin1String("text"), tr("Menu")).toString());
 	setMenu(m_menu);
 	setPopupMode(QToolButton::InstantPopup);
 	setButtonStyle(Qt::ToolButtonTextBesideIcon);
-	toolBarModified(ToolBarsManager::MenuBar);
+	handleActionsStateChanged({ActionsManager::ShowToolBarAction});
 
-	ToolBarWidget *toolBar(qobject_cast<ToolBarWidget*>(parent));
+	const MainWindow *mainWindow(MainWindow::findMainWindow(this));
+	const ToolBarWidget *toolBar(qobject_cast<ToolBarWidget*>(parent));
+
+	if (mainWindow)
+	{
+		connect(mainWindow, &MainWindow::arbitraryActionsStateChanged, this, &MenuButtonWidget::handleActionsStateChanged);
+	}
 
 	if (toolBar)
 	{
-		disconnect(toolBar, SIGNAL(buttonStyleChanged(Qt::ToolButtonStyle)), this, SLOT(setButtonStyle(Qt::ToolButtonStyle)));
+		disconnect(toolBar, &ToolBarWidget::buttonStyleChanged, this, &MenuButtonWidget::setButtonStyle);
 	}
 
-	connect(ToolBarsManager::getInstance(), SIGNAL(toolBarModified(int)), this, SLOT(toolBarModified(int)));
-	connect(m_menu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));
+	connect(m_menu, &Menu::aboutToShow, this, &MenuButtonWidget::updateMenu);
 }
 
-void MenuButtonWidget::toolBarModified(int identifier)
+void MenuButtonWidget::handleActionsStateChanged(const QVector<int> &identifiers)
 {
-	if (identifier == ToolBarsManager::MenuBar)
+	if (identifiers.contains(ActionsManager::ShowToolBarAction))
 	{
-		m_isHidden = (ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).normalVisibility == ToolBarsManager::AlwaysVisibleToolBar);
+		const MainWindow *mainWindow(MainWindow::findMainWindow(this));
+
+		m_isHidden = (mainWindow && mainWindow->getActionState(ActionsManager::ShowToolBarAction, {{QLatin1String("toolBar"), ToolBarsManager::MenuBar}}).isChecked);
 
 		updateGeometry();
 	}
@@ -63,11 +68,11 @@ void MenuButtonWidget::toolBarModified(int identifier)
 
 void MenuButtonWidget::updateMenu()
 {
-	disconnect(m_menu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));
+	disconnect(m_menu, &Menu::aboutToShow, this, &MenuButtonWidget::updateMenu);
 
 	m_menu->load(QLatin1String("menu/menuButton.json"));
 
-	connect(m_menu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));
+	connect(m_menu, &Menu::aboutToShow, this, &MenuButtonWidget::updateMenu);
 }
 
 QSize MenuButtonWidget::minimumSizeHint() const
