@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2016 - 2017 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
@@ -41,6 +41,7 @@
 #include <QtGui/QGuiApplication>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPainter>
+#include <QtGui/QPainterPath>
 #include <QtGui/QPixmapCache>
 #include <QtWidgets/QDesktopWidget>
 #include <QtWidgets/QGridLayout>
@@ -64,8 +65,10 @@ TileDelegate::TileDelegate(QObject *parent) : QStyledItemDelegate(parent),
 
 void TileDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
+	const QPalette::ColorGroup colorGroup(index.data(StartPageModel::IsEmptyRole).toBool() ? QPalette::Disabled : QPalette::Active);
 	const int textHeight(qRound(option.fontMetrics.boundingRect(QLatin1String("X")).height() * 1.5));
 	const bool isAddTile(index.data(Qt::AccessibleDescriptionRole).toString() == QLatin1String("add"));
+	const bool isDragged(index.data(StartPageModel::IsDraggedRole).toBool());
 	QRect rectangle(option.rect);
 	rectangle.adjust(3, 3, -3, -3);
 
@@ -74,17 +77,24 @@ void TileDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
 
 	painter->setRenderHint(QPainter::HighQualityAntialiasing);
 
-	if (isAddTile || index.data(StartPageModel::IsDraggedRole).toBool())
+	if (!isDragged && (option.state.testFlag(QStyle::State_MouseOver) || option.state.testFlag(QStyle::State_HasFocus)))
 	{
-		if (isAddTile && (option.state.testFlag(QStyle::State_MouseOver) || option.state.testFlag(QStyle::State_HasFocus)))
+		QColor highlightColor(QGuiApplication::palette().color(colorGroup, QPalette::Highlight));
+
+		if (option.state.testFlag(QStyle::State_MouseOver))
 		{
-			painter->setPen(QPen(QGuiApplication::palette().color(QPalette::Highlight), 3));
-		}
-		else
-		{
-			painter->setPen(QPen(QColor(26, 35, 126, 51), 1));
+			highlightColor.setAlpha(150);
 		}
 
+		painter->setPen(QPen(highlightColor, 3));
+	}
+	else
+	{
+		painter->setPen(QPen(QColor(26, 35, 126, 51), 1));
+	}
+
+	if (isAddTile || isDragged)
+	{
 		if (isAddTile)
 		{
 			painter->setBrush(QColor(179, 229, 252, 224));
@@ -133,10 +143,12 @@ void TileDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
 
 				break;
 			case ThumbnailBackground:
+				painter->save();
 				painter->setBrush(Qt::white);
 				painter->setPen(Qt::transparent);
 				painter->drawRect(rectangle);
 				painter->drawPixmap(rectangle, QPixmap(StartPageModel::getThumbnailPath(index.data(BookmarksModel::IdentifierRole).toULongLong())), QRect(0, 0, rectangle.width(), rectangle.height()));
+				painter->restore();
 
 				break;
 			default:
@@ -162,7 +174,9 @@ void TileDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
 	palette.setColor(QPalette::Text, QColor(26, 35, 128));
 
 	painter->setClipping(false);
-	painter->setPen(palette.color((index.flags().testFlag(Qt::ItemIsEnabled) ? QPalette::Active : QPalette::Disabled), QPalette::Text));
+	painter->setBrush(Qt::transparent);
+	painter->drawPath(path);
+	painter->setPen(palette.color(colorGroup, QPalette::Text));
 
 	if (m_mode == NoBackground)
 	{
@@ -172,18 +186,6 @@ void TileDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
 	{
 		painter->drawText(QRect(rectangle.x(), (rectangle.y() + rectangle.height()), rectangle.width(), textHeight), Qt::AlignCenter, option.fontMetrics.elidedText(index.data(Qt::DisplayRole).toString(), option.textElideMode, (rectangle.width() - 20)));
 	}
-
-	if (option.state.testFlag(QStyle::State_MouseOver) || option.state.testFlag(QStyle::State_HasFocus))
-	{
-		painter->setPen(QPen(QGuiApplication::palette().color(QPalette::Highlight), 3));
-	}
-	else
-	{
-		painter->setPen(QPen(QColor(26, 35, 126, 51), 1));
-	}
-
-	painter->setBrush(Qt::transparent);
-	painter->drawPath(path);
 }
 
 void TileDelegate::handleOptionChanged(int identifier, const QVariant &value)
@@ -237,6 +239,16 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 		return;
 	}
 
+	const QString key(getPixmapCacheKey());
+	QPixmap cachedPixmap;
+
+	if (QPixmapCache::find(key, &cachedPixmap))
+	{
+		painter.drawPixmap(contentsRect(), cachedPixmap);
+
+		return;
+	}
+
 	const QPixmap pixmap(m_path);
 
 	if (pixmap.isNull())
@@ -247,34 +259,10 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 	switch (m_mode)
 	{
 		case BestFitBackground:
-			{
-				const QString key(QLatin1String("start-page-best-fit-") + QString::number(width()) + QLatin1Char('-') + QString::number(height()));
-				const QPixmap *cachedBackground(QPixmapCache::find(key));
+			cachedPixmap = pixmap.scaled(size(), Qt::KeepAspectRatioByExpanding, Qt::SmoothTransformation);
+			cachedPixmap = cachedPixmap.copy(contentsRect().translated(((cachedPixmap.width() - width()) / 2), ((cachedPixmap.height() - height()) / 2)));
 
-				if (cachedBackground)
-				{
-					painter.drawPixmap(contentsRect(), *cachedBackground, contentsRect().translated(((cachedBackground->width() - width()) / 2), ((cachedBackground->height() - height()) / 2)));
-				}
-				else
-				{
-					const qreal pixmapAspectRatio(pixmap.width() / static_cast<qreal>(pixmap.height()));
-					const qreal backgroundAspectRatio(width() / static_cast<qreal>(height()));
-					QPixmap newBackground(size());
-
-					if (pixmapAspectRatio > backgroundAspectRatio)
-					{
-						newBackground = pixmap.scaledToHeight(height(), Qt::SmoothTransformation);
-					}
-					else
-					{
-						newBackground = pixmap.scaledToWidth(width(), Qt::SmoothTransformation);
-					}
-
-					painter.drawPixmap(contentsRect(), newBackground, contentsRect().translated(((newBackground.width() - width()) / 2), ((newBackground.height() - height()) / 2)));
-
-					QPixmapCache::insert(key, newBackground);
-				}
-			}
+			painter.drawPixmap(contentsRect(), cachedPixmap);
 
 			break;
 		case CenterBackground:
@@ -282,23 +270,9 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 
 			break;
 		case StretchBackground:
-			{
-				const QString key(QLatin1String("start-page-stretch-") + QString::number(width()) + QLatin1Char('-') + QString::number(height()));
-				const QPixmap *cachedBackground(QPixmapCache::find(key));
+			cachedPixmap = pixmap.scaled(size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
 
-				if (cachedBackground)
-				{
-					painter.drawPixmap(contentsRect(), *cachedBackground, contentsRect());
-				}
-				else
-				{
-					const QPixmap newBackground(pixmap.scaled(size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
-
-					painter.drawPixmap(contentsRect(), newBackground, contentsRect());
-
-					QPixmapCache::insert(key, newBackground);
-				}
-			}
+			painter.drawPixmap(contentsRect(), cachedPixmap);
 
 			break;
 		case TileBackground:
@@ -308,6 +282,11 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 		default:
 			break;
 	}
+
+	if (!cachedPixmap.isNull())
+	{
+		QPixmapCache::insert(key, cachedPixmap);
+	}
 }
 
 void StartPageContentsWidget::setBackgroundMode(StartPageContentsWidget::BackgroundMode mode)
@@ -321,6 +300,27 @@ void StartPageContentsWidget::setBackgroundMode(StartPageContentsWidget::Backgro
 	update();
 }
 
+QString StartPageContentsWidget::getPixmapCacheKey() const
+{
+	QString prefix;
+
+	switch (m_mode)
+	{
+		case BestFitBackground:
+			prefix = QLatin1String("start-page-best-fit");
+
+			break;
+		case StretchBackground:
+			prefix = QLatin1String("start-page-stretch");
+
+			break;
+		default:
+			break;
+	}
+
+	return (prefix.isEmpty() ? QString() : prefix + QLatin1Char('-') + QString::number(width()) + QLatin1Char('-') + QString::number(height()));
+}
+
 StartPageWidget::StartPageWidget(Window *parent) : QScrollArea(parent),
 	m_window(parent),
 	m_contentsWidget(new StartPageContentsWidget(this)),
@@ -522,13 +522,13 @@ void StartPageWidget::triggerAction(int identifier, const QVariantMap &parameter
 			return;
 	}
 
-	const QUrl url(m_currentIndex.data(BookmarksModel::UrlRole).toUrl());
+	const BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(m_currentIndex));
 
-	if (url.isValid())
+	if (bookmark)
 	{
 		m_urlOpenTime = QTime::currentTime();
 
-		Application::triggerAction(ActionsManager::OpenUrlAction, {{QLatin1String("url"), url}, {QLatin1String("hints"), QVariant(hints)}}, parentWidget(), trigger);
+		Application::triggerAction(ActionsManager::OpenBookmarkAction, {{QLatin1String("bookmark"), bookmark->getIdentifier()}, {QLatin1String("hints"), QVariant(hints)}}, parentWidget(), trigger);
 	}
 }
 
@@ -598,7 +598,7 @@ void StartPageWidget::openTile()
 
 	if (type == BookmarksModel::FolderBookmark)
 	{
-		const BookmarksModel::Bookmark *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
+		const BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(m_currentIndex));
 
 		if (bookmark && bookmark->rowCount() > 0)
 		{
@@ -627,7 +627,7 @@ void StartPageWidget::openTile()
 
 void StartPageWidget::editTile()
 {
-	BookmarksModel::Bookmark *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
+	BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(m_currentIndex));
 
 	if (bookmark)
 	{
@@ -646,11 +646,11 @@ void StartPageWidget::reloadTile()
 
 		if (path.isEmpty())
 		{
-			m_spinnerAnimation = new SpinnerAnimation(this);
+			m_spinnerAnimation = new SpinnerAnimation(QCoreApplication::instance());
 		}
 		else
 		{
-			m_spinnerAnimation = new GenericAnimation(path, this);
+			m_spinnerAnimation = new GenericAnimation(path, QCoreApplication::instance());
 		}
 
 		m_spinnerAnimation->start();
@@ -663,7 +663,7 @@ void StartPageWidget::reloadTile()
 
 void StartPageWidget::removeTile()
 {
-	BookmarksModel::Bookmark *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
+	BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(m_currentIndex));
 
 	if (bookmark)
 	{
@@ -944,7 +944,7 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 			{
 				case BookmarksModel::FolderBookmark:
 					{
-						const BookmarksModel::Bookmark *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
+						const BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(m_currentIndex));
 
 						if (bookmark && bookmark->rowCount() > 0)
 						{
@@ -1015,7 +1015,7 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 
 				if (type == BookmarksModel::FolderBookmark)
 				{
-					const BookmarksModel::Bookmark *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
+					const BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(m_currentIndex));
 
 					if (bookmark && bookmark->rowCount() > 0)
 					{
@@ -1080,13 +1080,34 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 	{
 		const QHelpEvent *helpEvent(static_cast<QHelpEvent*>(event));
 		const QModelIndex index(m_listView->indexAt(helpEvent->pos()));
-		const BookmarksModel::Bookmark *bookmark(BookmarksManager::getModel()->getBookmark(index.data(BookmarksModel::IdentifierRole).toULongLong()));
+		QString toolTip;
 
-		if (bookmark)
+		if (index.isValid())
 		{
-			const QKeySequence shortcut(ActionsManager::getActionShortcut(ActionsManager::OpenBookmarkAction, {{QLatin1String("startPageTile"), (index.row() + 1)}}));
+			if (index.data(Qt::AccessibleDescriptionRole).toString() == QLatin1String("add"))
+			{
+				toolTip = index.data(Qt::ToolTipRole).toString();
+			}
+			else
+			{
+				const BookmarksModel::Bookmark *bookmark(StartPageModel::getBookmark(index));
 
-			QToolTip::showText(helpEvent->globalPos(), QFontMetrics(QToolTip::font()).elidedText(bookmark->getTitle() + (shortcut.isEmpty() ? QString() : QLatin1String(" (") + shortcut.toString(QKeySequence::NativeText) + QLatin1Char(')')), Qt::ElideRight, (QApplication::desktop()->screenGeometry(m_listView).width() / 2)), m_listView, m_listView->visualRect(index));
+				if (bookmark)
+				{
+					const QKeySequence shortcut(ActionsManager::getActionShortcut(ActionsManager::OpenBookmarkAction, {{QLatin1String("startPageTile"), (index.row() + 1)}}));
+
+					toolTip = bookmark->getTitle() + (shortcut.isEmpty() ? QString() : QLatin1String(" (") + shortcut.toString(QKeySequence::NativeText) + QLatin1Char(')'));
+				}
+			}
+		}
+
+		if (toolTip.isEmpty())
+		{
+			QToolTip::hideText();
+		}
+		else
+		{
+			QToolTip::showText(helpEvent->globalPos(), QFontMetrics(QToolTip::font()).elidedText(toolTip, Qt::ElideRight, (QApplication::desktop()->screenGeometry(m_listView).width() / 2)), m_listView, m_listView->visualRect(index));
 		}
 
 		return true;
