@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2017 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -33,7 +34,7 @@ namespace Ui
 	class CertificateDialog;
 }
 
-class CertificateDialog : public Dialog
+class CertificateDialog final : public Dialog
 {
 	Q_OBJECT
 
@@ -58,13 +59,21 @@ public:
 		DigestSha256Field
 	};
 
-	explicit CertificateDialog(QList<QSslCertificate> certificates, QWidget *parent = nullptr);
+	enum DataRole
+	{
+		CertificateFieldRole = Qt::UserRole,
+		CertificateIndexRole,
+		ExtensionIndexRole,
+		ExtensionNameRole
+	};
+
+	explicit CertificateDialog(QVector<QSslCertificate> certificates, QWidget *parent = nullptr);
 	~CertificateDialog();
 
 protected:
-	void changeEvent(QEvent *event);
-	QStandardItem* createField(CertificateField field, QStandardItem *parent = nullptr, const QMap<int, QVariant> &data = QMap<int, QVariant>());
-	QString formatHex(const QString &source, const QChar &separator = QLatin1Char(' '));
+	void changeEvent(QEvent *event) override;
+	QStandardItem* createField(CertificateField field, QStandardItem *parent = nullptr, const QMap<int, QVariant> &metaData = {});
+	static QString formatHex(const QString &source, const QChar &separator = QLatin1Char(' '));
 
 protected slots:
 	void exportCertificate();
@@ -72,7 +81,7 @@ protected slots:
 	void updateValue();
 
 private:
-	QList<QSslCertificate> m_certificates;
+	QVector<QSslCertificate> m_certificates;
 	Ui::CertificateDialog *m_ui;
 };
 
