@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 - 2017 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,10 @@ class ComboBoxWidget : public QComboBox
 public:
 	explicit ComboBoxWidget(QWidget *parent = nullptr);
 
+	void setCurrentIndex(const QModelIndex &index);
+	void setCurrentIndex(int index);
 	ItemViewWidget* getView() const;
+	bool event(QEvent *event) override;
 
 private:
 	ItemViewWidget *m_view;
