@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +9,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -73,7 +73,7 @@ ContentsDialog::ContentsDialog(const QIcon &icon, const QString &title, const QS
 	titleLabel->setFont(font);
 
 	m_closeLabel->setToolTip(tr("Close"));
-	m_closeLabel->setPixmap(ThemesManager::getIcon(QLatin1String("window-close")).pixmap(16, 16));
+	m_closeLabel->setPixmap(ThemesManager::createIcon(QLatin1String("window-close")).pixmap(16, 16));
 	m_closeLabel->setMargin(5);
 	m_closeLabel->installEventFilter(this);
 
@@ -146,12 +146,12 @@ ContentsDialog::ContentsDialog(const QIcon &icon, const QString &title, const QS
 
 			scrollLayout->addWidget(m_payloadWidget);
 
-			QDialog *dialog(qobject_cast<QDialog*>(m_payloadWidget));
+			const QDialog *dialog(qobject_cast<QDialog*>(m_payloadWidget));
 
 			if (dialog)
 			{
-				connect(this, SIGNAL(accepted()), dialog, SLOT(accept()));
-				connect(this, SIGNAL(rejected()), dialog, SLOT(reject()));
+				connect(this, &ContentsDialog::accepted, dialog, &QDialog::accept);
+				connect(this, &ContentsDialog::rejected, dialog, &QDialog::reject);
 				connect(dialog, &QDialog::finished, [&](int result)
 				{
 					m_isAccepted = (result == QDialog::Accepted);
@@ -173,7 +173,7 @@ ContentsDialog::ContentsDialog(const QIcon &icon, const QString &title, const QS
 
 		m_contentsLayout->addWidget(m_buttonBox);
 
-		connect(m_buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(handleButtonClick(QAbstractButton*)));
+		connect(m_buttonBox, &QDialogButtonBox::clicked, this, &ContentsDialog::handleButtonClick);
 	}
 
 	setContextMenuPolicy(Qt::PreventContextMenu);
@@ -198,16 +198,18 @@ void ContentsDialog::showEvent(QShowEvent *event)
 
 void ContentsDialog::closeEvent(QCloseEvent *event)
 {
+	const bool isChecked(getCheckBoxState());
+
 	if (m_isAccepted)
 	{
-		emit accepted();
+		emit accepted(isChecked);
 	}
 	else
 	{
-		emit rejected();
+		emit rejected(isChecked);
 	}
 
-	emit finished();
+	emit finished((m_isAccepted ? QDialog::Accepted : QDialog::Rejected), isChecked);
 
 	event->accept();
 }
@@ -227,7 +229,7 @@ void ContentsDialog::keyPressEvent(QKeyEvent *event)
 			case Qt::Key_Enter:
 			case Qt::Key_Return:
 				{
-					QList<QPushButton*> buttons(findChildren<QPushButton*>());
+					const QList<QPushButton*> buttons(findChildren<QPushButton*>());
 
 					for (int i = 0; i < buttons.count(); ++i)
 					{
@@ -272,9 +274,13 @@ void ContentsDialog::handleButtonClick(QAbstractButton *button)
 
 void ContentsDialog::updateSize()
 {
+	m_headerWidget->setMaximumWidth(0);
+
 	adjustSize();
 	resize(sizeHint());
 
+	m_headerWidget->setMaximumWidth(QWIDGETSIZE_MAX);
+
 	if (parentWidget())
 	{
 		if (height() >= parentWidget()->height())
@@ -301,7 +307,11 @@ void ContentsDialog::setCheckBox(const QString &text, bool state)
 	m_checkBox->setText(text);
 	m_checkBox->setChecked(state);
 
+	m_headerWidget->setMaximumWidth(0);
+
 	adjustSize();
+
+	m_headerWidget->setMaximumWidth(QWIDGETSIZE_MAX);
 }
 
 bool ContentsDialog::getCheckBoxState() const
@@ -320,41 +330,36 @@ bool ContentsDialog::eventFilter(QObject *object, QEvent *event)
 	{
 		if (event->type() == QEvent::MouseButtonPress)
 		{
-			QMouseEvent *mouseEvent(static_cast<QMouseEvent*>(event));
-
-			m_offset = mouseEvent->pos();
+			m_offset = static_cast<QMouseEvent*>(event)->pos();
 		}
 		else if (event->type() == QEvent::MouseMove)
 		{
-			QMouseEvent *mouseEvent(static_cast<QMouseEvent*>(event));
-
-			move(mapToParent(mouseEvent->pos()) - m_offset);
+			move(mapToParent(static_cast<QMouseEvent*>(event)->pos()) - m_offset);
 		}
 	}
-	else if (object == m_closeLabel && event->type() == QEvent::MouseButtonPress)
+	else if (object == m_closeLabel && event->type() == QEvent::MouseButtonPress && static_cast<QMouseEvent*>(event)->button() == Qt::LeftButton)
 	{
-		QMouseEvent *mouseEvent(static_cast<QMouseEvent*>(event));
-
-		if (mouseEvent && mouseEvent->button() == Qt::LeftButton)
-		{
-			close();
-		}
+		close();
 	}
 	else if (event->type() == QEvent::KeyPress)
 	{
-		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
-
-		if ((keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return))
+		switch (static_cast<QKeyEvent*>(event)->key())
 		{
-			m_isAccepted = true;
+			case Qt::Key_Enter:
+			case Qt::Key_Return:
+				m_isAccepted = true;
 
-			close();
-		}
-		else if (keyEvent->key() == Qt::Key_Escape)
-		{
-			m_isAccepted = false;
+				close();
 
-			close();
+				break;
+			case Qt::Key_Escape:
+				m_isAccepted = false;
+
+				close();
+
+				break;
+			default:
+				break;
 		}
 	}
 
