@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2014 - 2015 Piotr Wójcik <chocimier@tlen.pl>
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -32,7 +32,7 @@ namespace Ui
 	class ImportDialog;
 }
 
-class ImportDialog : public Dialog
+class ImportDialog final : public Dialog
 {
 	Q_OBJECT
 
@@ -44,10 +44,14 @@ public:
 protected:
 	explicit ImportDialog(Importer *importer, QWidget *parent);
 
-	void changeEvent(QEvent *event);
+	void closeEvent(QCloseEvent *event) override;
+	void changeEvent(QEvent *event) override;
 
 protected slots:
-	void import();
+	void handleImportRequested();
+	void handleImportStarted(Importer::ImportType type, int total);
+	void handleImportProgress(Importer::ImportType type, int total, int amount);
+	void handleImportFinished(Importer::ImportType type, Importer::ImportResult result, int total);
 	void setPath(const QString &path);
 
 private:
