@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,20 +9,18 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
 #include "MenuBarWidget.h"
 #include "MainWindow.h"
-#include "WorkspaceWidget.h"
 #include "Menu.h"
 #include "ToolBarWidget.h"
-#include "../core/ActionsManager.h"
 #include "../core/SessionsManager.h"
 
 #include <QtCore/QFile>
@@ -39,20 +37,20 @@ namespace Otter
 
 MenuBarWidget::MenuBarWidget(MainWindow *parent) : QMenuBar(parent),
 	m_mainWindow(parent),
-	m_leftToolBar(NULL),
-	m_rightToolBar(NULL)
+	m_leftToolBar(nullptr),
+	m_rightToolBar(nullptr)
 {
 	QFile file(SessionsManager::getReadableDataPath(QLatin1String("menu/menuBar.json")));
-	file.open(QFile::ReadOnly);
+	file.open(QIODevice::ReadOnly);
 
-	const QJsonArray definition = QJsonDocument::fromJson(file.readAll()).array();
+	const QJsonArray definition(QJsonDocument::fromJson(file.readAll()).array());
 
 	file.close();
 
 	for (int i = 0; i < definition.count(); ++i)
 	{
-		const QJsonObject object = definition.at(i).toObject();
-		Menu *menu = new Menu(Menu::getRole(object.value(QLatin1String("identifier")).toString()), this);
+		const QJsonObject object(definition.at(i).toObject());
+		Menu *menu(new Menu(Menu::getMenuRoleIdentifier(object.value(QLatin1String("identifier")).toString()), this));
 		menu->load(object);
 
 		addMenu(menu);
@@ -60,9 +58,15 @@ MenuBarWidget::MenuBarWidget(MainWindow *parent) : QMenuBar(parent),
 
 	if (!isNativeMenuBar())
 	{
-		setup();
+		reload();
 
-		connect(ToolBarsManager::getInstance(), SIGNAL(toolBarModified(int)), this, SLOT(toolBarModified(int)));
+		connect(ToolBarsManager::getInstance(), &ToolBarsManager::toolBarModified, this, [&](int identifier)
+		{
+			if (identifier == ToolBarsManager::MenuBar)
+			{
+				reload();
+			}
+		});
 	}
 }
 
@@ -72,7 +76,7 @@ void MenuBarWidget::changeEvent(QEvent *event)
 
 	if (event->type() == QEvent::LanguageChange)
 	{
-		QTimer::singleShot(100, this, SLOT(updateSize()));
+		QTimer::singleShot(100, this, &MenuBarWidget::updateGeometries);
 	}
 }
 
@@ -80,20 +84,21 @@ void MenuBarWidget::resizeEvent(QResizeEvent *event)
 {
 	QMenuBar::resizeEvent(event);
 
-	updateSize();
+	updateGeometries();
 }
 
 void MenuBarWidget::contextMenuEvent(QContextMenuEvent *event)
 {
-	QMenu *menu = ToolBarWidget::createCustomizationMenu(ToolBarsManager::MenuBar);
+	QMenu *menu(ToolBarWidget::createCustomizationMenu(ToolBarsManager::MenuBar));
 	menu->exec(event->globalPos());
 	menu->deleteLater();
 }
 
-void MenuBarWidget::setup()
+void MenuBarWidget::reload()
 {
-	const ToolBarsManager::ToolBarDefinition definition = ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar);
+	const ToolBarsManager::ToolBarDefinition definition(ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar));
 	QStringList actions;
+	actions.reserve(definition.entries.count());
 
 	for (int i = 0; i < definition.entries.count(); ++i)
 	{
@@ -105,52 +110,52 @@ void MenuBarWidget::setup()
 		if (m_leftToolBar)
 		{
 			m_leftToolBar->deleteLater();
-			m_leftToolBar = NULL;
+			m_leftToolBar = nullptr;
 
-			setCornerWidget(NULL, Qt::TopLeftCorner);
+			setCornerWidget(nullptr, Qt::TopLeftCorner);
 		}
 
 		if (m_rightToolBar)
 		{
 			m_rightToolBar->deleteLater();
-			m_rightToolBar = NULL;
+			m_rightToolBar = nullptr;
 
-			setCornerWidget(NULL, Qt::TopRightCorner);
+			setCornerWidget(nullptr, Qt::TopRightCorner);
 		}
 
 		return;
 	}
 
-	const int position = actions.indexOf(QLatin1String("MenuBarWidget"));
-	const bool needsLeftToolbar = (position != 0);
-	const bool needsRightToolbar = (position != (definition.entries.count() - 1));
+	const int position(actions.indexOf(QLatin1String("MenuBarWidget")));
+	const bool needsLeftToolbar(position != 0);
+	const bool needsRightToolbar(position != (definition.entries.count() - 1));
 
 	if (needsLeftToolbar && !m_leftToolBar)
 	{
-		m_leftToolBar = new ToolBarWidget(-1, m_mainWindow->getWorkspace()->getActiveWindow(), this);
+		m_leftToolBar = new ToolBarWidget(ToolBarsManager::MenuBar, m_mainWindow->getActiveWindow(), this);
 
 		setCornerWidget(m_leftToolBar, Qt::TopLeftCorner);
 	}
 	else if (!needsLeftToolbar && m_leftToolBar)
 	{
 		m_leftToolBar->deleteLater();
-		m_leftToolBar = NULL;
+		m_leftToolBar = nullptr;
 
-		setCornerWidget(NULL, Qt::TopLeftCorner);
+		setCornerWidget(nullptr, Qt::TopLeftCorner);
 	}
 
 	if (needsRightToolbar && !m_rightToolBar)
 	{
-		m_rightToolBar = new ToolBarWidget(-1, m_mainWindow->getWorkspace()->getActiveWindow(), this);
+		m_rightToolBar = new ToolBarWidget(ToolBarsManager::MenuBar, m_mainWindow->getActiveWindow(), this);
 
 		setCornerWidget(m_rightToolBar, Qt::TopRightCorner);
 	}
 	else if (!needsRightToolbar && m_rightToolBar)
 	{
 		m_rightToolBar->deleteLater();
-		m_rightToolBar = NULL;
+		m_rightToolBar = nullptr;
 
-		setCornerWidget(NULL, Qt::TopRightCorner);
+		setCornerWidget(nullptr, Qt::TopRightCorner);
 	}
 
 	ToolBarsManager::ToolBarDefinition leftDefinition(definition);
@@ -174,44 +179,44 @@ void MenuBarWidget::setup()
 		}
 	}
 
-	if (m_leftToolBar)
+	const int menuBarHeight(actionGeometry(this->actions().at(0)).height());
+
+	if (m_leftToolBar || m_rightToolBar)
 	{
-		m_leftToolBar->setDefinition(leftDefinition);
+		const int toolBarHeight((m_leftToolBar ? m_leftToolBar->getIconSize() : m_rightToolBar->getIconSize()) + 12);
+
+		if (m_leftToolBar)
+		{
+			m_leftToolBar->setDefinition(leftDefinition);
+		}
+
+		if (m_rightToolBar)
+		{
+			m_rightToolBar->setDefinition(rightDefinition);
+		}
+
+		setFixedHeight((toolBarHeight > menuBarHeight) ? toolBarHeight : menuBarHeight);
+	}
+	else
+	{
+		setFixedHeight(menuBarHeight);
 	}
 
-	if (m_rightToolBar)
-	{
-		m_rightToolBar->setDefinition(rightDefinition);
-	}
-
-	const int menuBarHeight = actionGeometry(this->actions().at(0)).height();
-	const int toolBarHeight = ((m_leftToolBar || m_rightToolBar) ? (definition.iconSize + 12) : 0);
-
-	setFixedHeight((toolBarHeight > 0 && toolBarHeight > menuBarHeight) ? toolBarHeight : menuBarHeight);
-
-	QTimer::singleShot(100, this, SLOT(updateSize()));
+	QTimer::singleShot(100, this, &MenuBarWidget::updateGeometries);
 }
 
-void MenuBarWidget::toolBarModified(int identifier)
-{
-	if (identifier == ToolBarsManager::MenuBar)
-	{
-		setup();
-	}
-}
-
-void MenuBarWidget::updateSize()
+void MenuBarWidget::updateGeometries()
 {
 	if (!m_leftToolBar && !m_rightToolBar)
 	{
 		return;
 	}
 
-	int size = 0;
+	int size(0);
 
 	if (actions().count() > 0)
 	{
-		size = ((style()->pixelMetric(QStyle::PM_MenuBarHMargin, 0, this) * 2) + (style()->pixelMetric(QStyle::PM_MenuBarItemSpacing, 0, this) * actions().count()));
+		size = ((style()->pixelMetric(QStyle::PM_MenuBarHMargin, nullptr, this) * 2) + (style()->pixelMetric(QStyle::PM_MenuBarItemSpacing, nullptr, this) * actions().count()));
 
 		for (int i = 0; i < actions().count(); ++i)
 		{
@@ -225,8 +230,8 @@ void MenuBarWidget::updateSize()
 
 	if (m_rightToolBar && width() > (size + (m_leftToolBar ? m_leftToolBar->sizeHint().width() : 0) + (m_rightToolBar ? m_rightToolBar->sizeHint().width() : 0)))
 	{
-		const int offset = (size - (m_leftToolBar ? m_leftToolBar->sizeHint().width() : 0));
-		ToolBarWidget *toolBar = qobject_cast<ToolBarWidget*>(m_rightToolBar);
+		const int offset(size - (m_leftToolBar ? m_leftToolBar->sizeHint().width() : 0));
+		ToolBarWidget *toolBar(qobject_cast<ToolBarWidget*>(m_rightToolBar));
 		toolBar->move(QPoint(offset, 0));
 		toolBar->resize((width() - offset), toolBar->height());
 	}
