@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
 * Copyright (C) 2015 Jan Bajer aka bajasoft <jbajer@gmail.com>
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -10,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -30,27 +30,24 @@ namespace Otter
 
 class Notification;
 
-class NotificationDialog : public QDialog
+class NotificationDialog final : public QFrame
 {
 	Q_OBJECT
 
 public:
 	explicit NotificationDialog(Notification *notification, QWidget *parent = nullptr);
 
-	bool eventFilter(QObject *object, QEvent *event);
-
 protected:
-	void closeEvent(QCloseEvent *event);
-	void resizeEvent(QResizeEvent *event);
-
-protected slots:
-	void aboutToClose();
-	void clean();
+	void changeEvent(QEvent *event) override;
+	void timerEvent(QTimerEvent *event) override;
+	void resizeEvent(QResizeEvent *event) override;
+	void mouseReleaseEvent(QMouseEvent *event) override;
 
 private:
 	Notification *m_notification;
 	QLabel *m_closeLabel;
 	QPropertyAnimation *m_animation;
+	int m_closeTimer;
 };
 
 }
