@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 - 2017 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -9,11 +10,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -33,7 +34,9 @@ namespace Ui
 	class ToolBarDialog;
 }
 
-class ToolBarDialog : public Dialog
+class OptionWidget;
+
+class ToolBarDialog final : public Dialog
 {
 	Q_OBJECT
 
@@ -41,29 +44,35 @@ public:
 	enum DataRole
 	{
 		IdentifierRole = Qt::UserRole,
-		OptionsRole
+		OptionsRole,
+		ParametersRole,
+		HasOptionsRole
 	};
 
-	explicit ToolBarDialog(const ToolBarsManager::ToolBarDefinition &definition = ToolBarsManager::ToolBarDefinition(), QWidget *parent = nullptr);
+	explicit ToolBarDialog(const ToolBarsManager::ToolBarDefinition &definition = {}, QWidget *parent = nullptr);
 	~ToolBarDialog();
 
 	ToolBarsManager::ToolBarDefinition getDefinition() const;
-	bool eventFilter(QObject *object, QEvent *event);
+	bool eventFilter(QObject *object, QEvent *event) override;
 
 protected:
-	void changeEvent(QEvent *event);
-	void addEntry(const ActionsManager::ActionEntryDefinition &entry, QStandardItem *parent = nullptr);
-	QStandardItem* createEntry(const QString &identifier, const QVariantMap &options = QVariantMap());
-	ActionsManager::ActionEntryDefinition getEntry(QStandardItem *item) const;
+	void changeEvent(QEvent *event) override;
+	void addEntry(const ToolBarsManager::ToolBarDefinition::Entry &entry, QStandardItem *parent = nullptr);
+	QStandardItem* createEntry(const QString &identifier, const QVariantMap &options = {}, const QVariantMap &parameters = {}) const;
+	ToolBarsManager::ToolBarDefinition::Entry getEntry(QStandardItem *item) const;
+	QMap<int, QVariant> createEntryData(const QString &identifier, const QVariantMap &options = {}, const QVariantMap &parameters = {}) const;
 
 protected slots:
-	void addEntry();
+	void addNewEntry();
 	void editEntry();
 	void addBookmark(QAction *action);
 	void restoreDefaults();
+	void showAvailableEntriesContextMenu(const QPoint &position);
+	void showCurrentEntriesContextMenu(const QPoint &position);
 	void updateActions();
 
 private:
+	QStandardItemModel *m_currentEntriesModel;
 	ToolBarsManager::ToolBarDefinition m_definition;
 	Ui::ToolBarDialog *m_ui;
 };
