@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2014 - 2016 Piotr Wójcik <chocimier@tlen.pl>
 * Copyright (C) 2015 - 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
@@ -304,7 +304,7 @@ void WebsitePreferencesDialog::handleButtonClicked(QAbstractButton *button)
 
 						if (entryIndex.data(Qt::CheckStateRole).toBool())
 						{
-							contentBlockingProfiles.append(entryIndex.data(Qt::UserRole).toString());
+							contentBlockingProfiles.append(entryIndex.data(ContentFiltersManager::NameRole).toString());
 						}
 					}
 				}
@@ -414,7 +414,7 @@ void WebsitePreferencesDialog::updateContentBlockingProfile(const QString &name)
 		{
 			const QModelIndex entryIndex(m_ui->contentBlockingProfilesViewWidget->getIndex(j, 0, categoryIndex));
 
-			if (entryIndex.data(Qt::UserRole).toString() == name)
+			if (entryIndex.data(ContentFiltersManager::NameRole).toString() == name)
 			{
 				m_ui->contentBlockingProfilesViewWidget->setData(entryIndex, profile->getTitle(), Qt::DisplayRole);
 				m_ui->contentBlockingProfilesViewWidget->setData(entryIndex.sibling(j, 2), Utils::formatDateTime(profile->getLastUpdate()), Qt::DisplayRole);
