@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2019 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2014 - 2017 Jan Bajer aka bajasoft <jbajer@gmail.com>
 * Copyright (C) 2016 - 2017 Piotr Wójcik <chocimier@tlen.pl>
 *
@@ -257,7 +257,7 @@ PreferencesAdvancedPageWidget::PreferencesAdvancedPageWidget(QWidget *parent) :
 		}
 
 		const QList<QSslCipher> defaultCiphers(NetworkManagerFactory::getDefaultCiphers());
-		const QList<QSslCipher> supportedCiphers(QSslSocket::supportedCiphers());
+		const QList<QSslCipher> supportedCiphers(QSslConfiguration::supportedCiphers());
 
 		for (int i = 0; i < supportedCiphers.count(); ++i)
 		{
