@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2018 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2014 - 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
 * Copyright (C) 2014 Piotr Wójcik <chocimier@tlen.pl>
 *
@@ -11,11 +11,11 @@
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program.  If not, see <http://www.gnu.org/licenses/>.
+* along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 **************************************************************************/
 
@@ -32,7 +32,7 @@ namespace Ui
 	class PreferencesGeneralPageWidget;
 }
 
-class PreferencesGeneralPageWidget : public QWidget
+class PreferencesGeneralPageWidget final : public QWidget
 {
 	Q_OBJECT
 
@@ -40,15 +40,17 @@ public:
 	explicit PreferencesGeneralPageWidget(QWidget *parent = nullptr);
 	~PreferencesGeneralPageWidget();
 
+public slots:
+	void save();
+
 protected:
-	void changeEvent(QEvent *event);
+	void changeEvent(QEvent *event) override;
 
 protected slots:
 	void useCurrentAsHomePage();
 	void useBookmarkAsHomePage(QAction *action);
 	void restoreHomePage();
 	void setupAcceptLanguage();
-	void save();
 
 private:
 	QString m_acceptLanguage;
